from .argument import Argument

IS_DEBUG = True

APP = "trans"

EXIT = "exit()"
CLEAR = "clear()"
INFO = "info()"
OPEN = "open()"
HELP = "help()"
VALUE = "value()"
IMAGES = "images()"

CS_NAME = "cs"
FIRST_LANG = CS_NAME
SECOND_LANG = "en"
FIRST_LANG_NAME = "first"
SECOND_LANG_NAME = "second"
LANGUAGE_NAME = "languages"
SETTINGS_FILE = "settings.cfg"
URI_GOOGLE_TRANS = "https://translate.google.com/"

INPUT = ">>> "
LOADING = "..."
LAST_LINE = "\r"

ERROR_ICON = "[x]"
ARGUMENTS = [

    Argument( 'h', 'help', 'Print this help text and exit' ),
    Argument( 'm', 'message', 'The message is for translation.' ),
    Argument( 'f', 'first', 'Set a first language (example: "cs" or "en" ).' ),
    Argument( 's', 'second', 'Set a second language (example: "en" or "cs" ).' ),
    Argument( 'o', 'open', 'Open a web browser, with a translator tab.' ),
    Argument( 'v', 'value', 'Open a web browser, with a value word tab.' ),
    Argument( 'i', 'image', 'Open a web browser, with a images tab.' ),
]



#!/usr/bin/python

import sys, getopt
from . import constants
from .prints import Prints
from .translator import Translator
from .configuration import Configuration
from .thirdside import ThirdSide

class App:

    def __init__( self ):
        self.prints = Prints()
        self.translator = Translator( self )
        self.configuration = Configuration()
        self.thirdside = ThirdSide( self )


    def exit_app( self, value = 0 ):
        sys.exit( value )
        pass


    def set_vars_from_args( self, argv ):
        help = constants.ARGUMENTS[0]
        message = constants.ARGUMENTS[1]
        first = constants.ARGUMENTS[2]
        second = constants.ARGUMENTS[3]
        open = constants.ARGUMENTS[4]
        value = constants.ARGUMENTS[5]
        images = constants.ARGUMENTS[6]

        try:
            opts, args = getopt.getopt(argv,
            f"{ help.short }{open.short}{value.short}{images.short}{ message.short }:{ first.short }:{ second.short }:",
            [ help.long, {open.long}, {value.long}, {images.long}, f"{ message.long }=", f"{ first.long }=", f"{ second.long }=" ])

        except getopt.GetoptError:
            self.exit_app( 2 )

        for opt, arg in opts:
            if opt in help.get_full():
                prints.helper()
                self.exit_app()
            elif opt in open.get_full():
                open.set_value( True )
            elif opt in value.get_full():
                value.set_value( True )
            elif opt in images.get_full():
                images.set_value( True )
            elif opt in message.get_full():
                message.set_value( arg )
            elif opt in first.get_full():
                first.set_value( arg )
            elif opt in second.get_full():
                second.set_value( arg )


    def clear( self ):

        self.prints.clear()
        self.translator.clear()


    def checkFunction( self, text ):

        if ( text == constants.EXIT ):
            self.exit_app()
        elif ( text == constants.CLEAR ):
            self.clear()
            return True
        elif ( text == constants.INFO ):
            self.prints.print_informations()
            return True
        elif ( text == constants.OPEN ):
            self.thirdside.open_google_trans()
            return True
        elif ( text == constants.VALUE ):
            if self.thirdside.open_google():
                self.prints.print_nots_value()
            return True
        elif ( text == constants.IMAGES ):
            self.thirdside.open_google_images()
            return True
        elif ( text == constants.HELP ):
            self.prints.print_helps()
            return True
        
        return False


    def main( self ):
        argument = sys.argv[1:]
        self.set_vars_from_args(argument)
        self.configuration.check_languages()

        self.scenario( argument )


    def scenario( self, argument ):

        if ( not argument ):
            self.prints.print_informations()
            self.translator.translation_loop()
        else:
            message = constants.ARGUMENTS[1]
            open = constants.ARGUMENTS[4]
            value = constants.ARGUMENTS[5]
            images = constants.ARGUMENTS[6]

            if open.value or value.value or images.value:
                if message.value:
                    self.prints.debug( "set history" )
                    self.translator.history.set_text( message.value )
                    self.translator.detectDect( message.value )

            if open.value:
                self.prints.debug("open translator")
                self.checkFunction( constants.OPEN )
            else:
                self.prints.debug("translate")
                self.translator.translate( message.value )
            
            if value.value and not open.value:
                self.prints.debug("open value")
                self.checkFunction( constants.VALUE )
            elif value.value and open.value:
                self.prints.debug(f"{ constants.LAST_LINE }translate")
                self.translator.translate( message.value )
                self.prints.debug("open value")
                self.checkFunction( constants.VALUE )
            
            if images.value:
                self.prints.debug("open images")
                self.checkFunction( constants.IMAGES )



import pathlib
from . import constants
import configparser

class Configuration:

    def __init__( self ):

        self.config = configparser.ConfigParser()
        self.file_read = None
        self.open_config()


    def get_path_file( self ):
        dir = pathlib.Path(__file__).parent
        return (dir / constants.SETTINGS_FILE )


    def open_config( self ):
        self.file_read = self.config.read( self.get_path_file() )


    def write_config( self ):
        with open( self.get_path_file(), 'w+' ) as configfile:
            self.config.write(configfile)


    def has_langs_section( self ):
        return self.config.has_section( constants.LANGUAGE_NAME )


    def has_lang_option( self, option ):

        if not self.has_langs_section():
            self.config[ constants.LANGUAGE_NAME ] = {}

        return self.config.has_option( constants.LANGUAGE_NAME, option )


    def get_lang_config( self, option ):

        if self.has_lang_option( option ):
            return self.config.get( constants.LANGUAGE_NAME, option )
        
        return None
    

    def set_lang_config( self, option, value ):

        if self.has_lang_option( option ):
            return self.config.set( constants.LANGUAGE_NAME, option, value )
        else:
            self.config[ constants.LANGUAGE_NAME ][ option ] = value


    def get_first_lang_config( self ):
        return self.get_lang_config( constants.FIRST_LANG_NAME )


    def get_second_lang_config( self ):
        return self.get_lang_config( constants.SECOND_LANG_NAME )


    def set_first_lang_config( self, value = constants.FIRST_LANG):

        if value == None:
            value = constants.FIRST_LANG

        self.set_lang_config( constants.FIRST_LANG_NAME, value )


    def set_second_lang_config( self, value = None ):

        if value == None:
            value = constants.SECOND_LANG

        self.set_lang_config( constants.SECOND_LANG_NAME, value )
    

    def check_languages( self ):
        
        first = constants.ARGUMENTS[2]
        second = constants.ARGUMENTS[3]
        first_value = first.value
        second_value = second.value

        if self.file_read:

            if not first_value:
                first_value = self.get_first_lang_config()

            if not second_value:
                second_value = self.get_second_lang_config()

            self.set_first_lang_config(first_value)
            self.set_second_lang_config(second_value)
            self.write_config()

        else:

            if not first_value:
                first_value = constants.FIRST_LANG

            if not second_value:
                second_value = constants.SECOND_LANG

            self.set_first_lang_config(first_value)
            self.set_second_lang_config(second_value)
            self.write_config()

        first.set_value( first_value )
        second.set_value( second_value )


import sys, os
from . import constants

class Prints:

    def print_informations( self ):
        print( "Google translator (googletrans==4.0.0-rc1)" )


    def print_helps( self ):

        print( "| Functions for manipulating this an app." )
        print( "|" )
        print(f"| {'exit()' : <10} For exited this app.")
        print(f"| {'clear()' : <10} For clearing all a texts.")
        print(f"| {'open()' : <10} For opening a web google translator, with return an history word.")
        print(f"| {'value()' : <10} For opening a web google, with return an translated history word.")
        print(f"| {'images()' : <10} For opening a web google images, with return an history word.")


    def print_nots_value( self ):

        print( f"{ constants.ERROR_ICON } This a word not's value." )
    

    def print_error( self ):

        self.print_trans( f"{ constants.ERROR_ICON } This a word, not's exist in the google, in a list words." )


    def print_loading( self ):
        sys.stdout.write( constants.LOADING );
        sys.stdout.flush()


    def print_trans( self, text ):
        print( constants.LAST_LINE + text )
    

    def clear_console( self ):
        command = 'clear'
        if os.name in ('nt', 'dos'):  # If Machine is running on Windows, use cls
            command = 'cls'
        os.system(command)

    
    def clear( self ):
        self.clear_console()

    
    def helper( self ):
        print( f'Usage: { constants.APP } [OPTIONS]\n' )
        print(f"{'Option' : <30}Explanation")
        print(f"{'--------' : <30}-------")
        for argument in constants.ARGUMENTS:
            arguments = f'{ argument.get_short() }, { argument.get_long() }'
            print(f"{ arguments : <30}{argument.message}")
    

    def debug( self, value ):

        if constants.IS_DEBUG:
            print( value )


from . import constants
import webbrowser

class ThirdSide:

    def __init__( self, transfv ):
        self.transfv = transfv

    def open_google( self ):

        if ( self.transfv.translator.history.second_lang != constants.CS_NAME ):
            return True  # Just end this a method, and continue function for print a message.

        uri = self.get_uri_google( f"{ self.transfv.translator.history.text_trans }+v%C3%BDznam" )
        webbrowser.open( uri )
        return False
    
    def open_google_images( self ):

        arg = "&tbm=isch"
        uri = self.get_uri_google( self.transfv.translator.history.text, arg )
        webbrowser.open( uri )


    def open_google_trans( self ):

        uri = self.get_uri_google_trans()
        webbrowser.open( uri )


    def get_uri_google_trans( self ):

        self.transfv.translator.history.set_langs()

        text = self.transfv.translator.history.text
        if not text:
            text = ""

        return f"{ constants.URI_GOOGLE_TRANS }?sl={ self.transfv.translator.history.first_lang }&tl={ self.transfv.translator.history.second_lang }&text={ text }&op=translate"
    

    def get_uri_google( self, text = "", args = "" ):

        return f"https://www.google.com/search?q={ text }{ args }"


from . import constants
import googletrans

class History:

    def __init__( self ):

        self.text = ""
        self.text_trans = ""
        self.first_lang = ""
        self.second_lang = ""
    

    def set_langs( self ):

        if not self.text:
            self.first_lang = constants.ARGUMENTS[2].value
            self.second_lang = constants.ARGUMENTS[3].value


    def set_text( self, text ):

        self.text = text
    

    def set_text_trans( self, text ):

        self.text_trans = text


    def set_first_lang( self, lang ):

        self.first_lang = lang


    def set_second_lang( self, lang ):

        self.second_lang = lang



class Translator:
    
    def __init__( self, transfv ):
        self.dest = constants.FIRST_LANG
        self.translator = googletrans.Translator()
        self.history = History()
        self.transfv = transfv


    def clear( self ):

        self.history.__init__()


    def translation_loop( self ):

        while( True ):

            text = input( constants.INPUT )

            if self.transfv.checkFunction( text ):
                continue

            self.history.set_text( text )

            self.translate( text )


    def translate( self, text ):

        if ( not text ):
            return

        self.transfv.prints.print_loading()

        translate = ""
        try:
            dest = self.detectDect( text )
            translate = self.translator.translate( text, dest=dest ).text
        except:
            self.clear()
            self.transfv.prints.print_error()
            return

        self.transfv.prints.print_trans( translate )
        self.history.set_text_trans( translate )


    def detectDect( self, text ):

        dest = self.translator.detect( text ).lang
        self.history.set_first_lang( dest )

        first = constants.ARGUMENTS[2]
        second = constants.ARGUMENTS[3]

        if ( dest == first.value ):
            dest = second.value
        elif ( dest == second.value ):
            dest = first.value

        self.history.set_second_lang( dest )
        
        return dest



class Argument:

    def __init__( self, argShort, argLong, message = None ):
        self.short = argShort
        self.long = argLong
        self.message = message
        self.value = None
    

    def get_short( self ):
        return f"-{ self.short }"
    

    def get_long( self ):
        return f"--{ self.long }"
    

    def get_full( self ):
        return ( self.get_short(), self.get_long() )
    

    def set_value( self, value ):
        self.value = value


import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(

    name='transfv',
    version='1.0.7',
    description='Google translator for a terminal.',
    long_description=README,
    long_description_content_type="text/markdown",

    url='https://github.com/filipvrba/trans.git', 
    author='Filip Vrba',
    author_email='filipvrba@pm.me',
    license="MIT",
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    ],
    keywords=['python', 'google', 'translator', 'fv', 'transfv'],

    packages=find_packages(),
    install_requires=["googletrans==4.0.0-rc1"],

    entry_points={
        "console_scripts": [
            "trans=transfv.__init__:App.main",
        ]
    },
)
